//RS232 Lib
//C: AtMega32

//Includes
#include <avr/io.h>

//Wenn noch nicht getan, dann CPU Frequenz definieren
#ifndef F_CPU 
	#warning "F_CPU war noch nicht definiert. Nun F_CPU = 16 000 000"
	#define F_CPU 16000000UL
#endif

//Baudrate definieren
#define BAUD 9600UL

//UBRR Val berechnen
//dabei clever runden (siehe dazu:
//http://www.mikrocontroller.net/articles/AVR-Tutorial:_UART)
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)
//damit reale Baudrate berechnen
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))
//damit den Fehler in Promille berechnen
#define BAUD_ERROR ((BAUD_REAL*1000)/(BAUD))

//max +/- 10 Promille Fehler sonst abbruch
#if ((BAUD_ERROR > 1010) || (BAUD_ERROR < 990))
	#error "Baudratefehler grer als 1%, somit abbruch"
#endif




//Initialisiert den UART
void uart_init(void) {

	//UBRR Register schreiben
	UBRRH = UBRR_VAL >> 8;
	UBRRL = UBRR_VAL & 0xFF;
	
	//bertragungsmodus whlen
	//Rx und Tx einschalten
	//Data mode 8N1, asyncron
	UCSRB |= (1 << RXEN) | (1 << TXEN);
	UCSRC = (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);
}


//Zeichen senden
int uart_putc(unsigned char c) {
	//warten bis senden mglich
	while (!(UCSRA & (1<<UDRE))) {
   ;
	}

	//Zeichen senden
	UDR = c;

	return 1;
}

//String senden
void uart_puts (char *s) {
	//sl lange *s != '\0' ist (ungleich String-Endezeichen)
	while (*s) {
   	uart_putc(*s);
   	s++;
   }
}

//Zeichen empfangen
uint8_t uart_getc(void) {
	//warten bis Zeichen verfgbar
	while (!(UCSRA & (1<<RXC))) {
	;
	}

	//Zeichen aus UDR zurckgeben
   return UDR;
}
